<?php
namespace DaReactions;
use Exception;
class Api
{
    private $baseUrl = 'da/v1/reactions';
    public function __construct()
    {
    }
    public function addApiRoutes__premium_only(): void
    {
        /// Get all Reactions
        register_rest_route(
            $this->baseUrl,
            '/get-reactions/',
            [
                'methods' => 'GET',
                'callback' => [$this, 'listReactions'],
                'permission_callback' => [$this, 'allowAll'],
            ]
        );
        /// Get main reaction for content
        register_rest_route(
            $this->baseUrl,
            '/get-main-reaction-for-content/(?P<resource_type>[a-zA-Z0-9-]+)/(?P<resource_id>\d+)',
            [
                'methods' => 'GET',
                'callback' => [$this, 'getMainReactionForContent'],
                'permission_callback' => [$this, 'allowAll'],
            ]
        );
        /// Get reactions for content
        register_rest_route(
            $this->baseUrl,
            '/get-reactions-for-content/(?P<resource_type>[a-zA-Z0-9-]+)/(?P<resource_id>\d+)',
            [
                'methods' => 'GET',
                'callback' => [$this, 'getReactionsForContent'],
                'permission_callback' => [$this, 'allowAll'],
            ]
        );
        /// Get reactions for content
        register_rest_route(
            $this->baseUrl,
            '/get-reactions-and-users-for-content/(?P<resource_type>[a-zA-Z0-9-]+)/(?P<resource_id>\d+)',
            array(
                'methods'             => 'GET',
                'callback'            => array( $this, 'getReactionsAndUsersForContent' ),
                'permission_callback' => array( $this, 'allowAll' ),
                'args'                => array(
                    'reaction_id' => array(
                        'validate_callback' => function($param, $request, $key) {
                            return is_numeric( $param );
                        }
                    ),
                    'page_size'   => array(
                        'validate_callback' => function($param, $request, $key) {
                            return is_numeric( $param ) && $param > 0;
                        }
                    ),
                    'page_num'    => array(
                        'validate_callback' => function($param, $request, $key) {
                            return is_numeric( $param ) && $param > 0;
                        }
                    ),
                ),
            )
        );
    }
    public function allowAll(): bool
    {
        return true;
    }
    /**
     * @throws Exception
     */
    public function listReactions()
    {
        return rest_ensure_response(
            Data::getAllContentReactions()
        );
    }
    /**
     */
    public function getMainReactionForContent($data)
    {
        return rest_ensure_response(
            Data::getMainReactionForContent(
                $data['resource_id'],
                $data['resource_type']
            )
        );
    }
	public function getReactionsForContent($data)
    {
        return rest_ensure_response(
            array_map(
                static function ($r) {
                    return [
                        'ID' => $r->ID,
                        'label' => $r->label,
                        'file_name' => $r->file_name,
                        'created_at' => $r->created_at,
                        'color' => $r->color,
                        'active' => (bool) $r->active,
                        'sort_order' => $r->sort_order,
                        'total' => $r->total,
                        'percentage' => $r->percentage
                    ];
                },
                Data::getReactionsForContent(
                    $data['resource_id'],
                    $data['resource_type']
                )
            )
        );
    }
    /**
     */
    public function getReactionsAndUsersForContent($data)
    {
        return
            rest_ensure_response(
                Data::getReactionsAndUsersForContent(
                    $data['resource_id'],
                    $data['resource_type'],
                    $data['reaction_id'] ?? 0,
                    $data['page_size'] ?? 10,
                    $data['page_num'] ?? 1
            )
        );
    }
}
