<?php
/**
 * Class Gutenberg
 *
 * Creates the block for the Gutenberg Editor
 *
 * @package DaReactions\Plugins
 *
 * @since 1.2.0
 */
namespace DaReactions\Plugins;
use DaReactions\Data;
use DaReactions\Options;
use DaReactions\Utils;
use Freemius_Exception;
use JetBrains\PhpStorm\NoReturn;
/**
 * Class Gutenberg
 *
 * Creates the block for the Gutenberg Editor
 *
 * @package DaReactions\Editor
 *
 * @since 1.2.0
 */
class Gutenberg {
    /**
     * Enqueues Gutenberg’s block
     *
     * @since 1.2.0
     */
    public function enqueueBlockEditorAssets__premium_only() {
        global $post;
        if (!isset($post)) {
            return;
        }
        $options     = Options::getInstance( 'graphic' );
        $button_size = $options->getOption( 'button_size', 64 );
        $template    = $options->getOption( 'use_template', 'exposed' );
        wp_register_script(
            'da-reactions-gutenberg-block',
            DA_REACTIONS_URL . 'assets/dist/gutenberg.js',
            array( 'wp-blocks', 'wp-i18n', 'wp-element' )
        );
        $item_type = $post->post_type . '-g' . $post->ID;
        wp_localize_script(
            'da-reactions-gutenberg-block',
            'DaReactionsGutenbergBlock',
            array(
                'item_type'     => $item_type,
                'button_size'   => $button_size,
                'use_template'  => $template,
                'loader_url'    => DA_REACTIONS_URL . 'assets/dist/loading.svg',
                'preview_image' => admin_url( 'admin-ajax.php?action=da_reactions_preview_image' ),
                'nonce'         => wp_create_nonce( 'da-reactions-gutenberg' ),
            )
        );
        register_block_type( 'da-reactions/gutenberg-block', array(
            'editor_script' => 'da-reactions-gutenberg-block'
        ) );
    }
    /**
     * Clear database from unused blocks on post save
     *
     * @param $post_id
     *
     * @return bool
     */
    public function cleanDatabaseOnPostSave__premium_only( $post_id ) {
        if ( wp_is_post_revision( $post_id ) ) {
            return $post_id;
        }
        $post = get_post( $post_id );
        $resource_type = $post->post_type . '-g' . $post_id;
        $content = $post->post_content;
        $start_pattern = '<!-- wp:da-reactions/gutenberg-block -->';
        $end_pattern   = '<!-- /wp:da-reactions/gutenberg-block -->';
        $gutenberg_blocks = Utils::getContentsBetween( $content, $start_pattern, $end_pattern );
        $gutenberg_block_ids = array();
        foreach ( $gutenberg_blocks as $gutenberg_block ) {
            $parsed = array();
            preg_match( '/data-id="([^"]*)"/i', $gutenberg_block, $parsed );
            if ( count( $parsed ) > 1 ) {
                $gutenberg_block_ids[] = (int) $parsed[1];
            }
        }
        return Data::deleteGutenbergBlockVotes( $resource_type, $gutenberg_block_ids ) !== false;
    }
    /**
     * Generates a preview image to be displayed in Gutenberg block
     * @throws Freemius_Exception
     */
    public function previewImage(): void
    {
        $reactions   = Data::getAllReactions();
        $options     = Options::getInstance( 'graphic' );
        $button_size = $options->getOption( 'button_size', 64 );
        $image         = wp_imagecreatetruecolor( ( $button_size + 8 ) * count( $reactions ), ( $button_size + 8 ) );
        $alpha_channel = imagecolorallocatealpha( $image, 0, 0, 0, 127 );
        imagecolortransparent( $image, $alpha_channel );
        imagefill( $image, 0, 0, $alpha_channel );
        $center_x = ( $button_size / 2 ) + 4;
        $center_y = $center_x;
        imageantialias( $image, true );
        foreach ( $reactions as $reaction ) {
            $color = Utils::hexColorAllocate( $image, $reaction->color );
            imagefilledellipse( $image, $center_x, $center_y, $button_size, $button_size, $color );
            imagefilledellipse( $image, $center_x, $center_y, $button_size - 8, $button_size - 8, $alpha_channel );
            $center_x += $button_size + 8;
        }
        header( "Content-type:image/png" );
        imagepng( $image );
        imagedestroy( $image );
        exit();
    }
}
