<?php
/**
 * Class Network
 *
 * Manages network related tasks
 *
 * @package DaReactions
 *
 * @since 1.0.0
 */
namespace DaReactions\Plugins;
use DaReactions\Data;
use DaReactions\FileSystem;
use DaReactions\Options;
/**
 * Class Network
 *
 * Manages network related tasks
 *
 * @package DaReactions
 *
 * @since 1.0.0
 */
class Network {
    /**
     * @var string $plugin_name
     * The name of the plugin
     *
     * @since 1.0.0
     */
    private $plugin_name;
    /**
     * Network constructor.
     *
     * @param string $plugin_name
     *
     * @since 1.0.0
     */
    public function __construct( $plugin_name ) {
        $this->plugin_name = $plugin_name;
    }
    /**
     * Initializes default data for a new blog
     *
     * Attached to wpmu_new_blog hook
     *
     * @param integer $blog_id
     *
     * @since 1.0.0
     */
    public function createDataForNewBlog__premium_only( $blog_id ) {
        // Database
        Data::createReactionsTable( );
        Data::createVotesTable( );
        Data::createDefaultReactions( );
        // Options
        Options::createInstance( $this->plugin_name . '_buttons', 'buttons', $blog_id );
        Options::createInstance( $this->plugin_name . '_general', 'general', $blog_id );
        Options::createInstance( $this->plugin_name . '_graphic', 'graphic', $blog_id );
        Options::createInstance( $this->plugin_name . '_notices', 'notices', $blog_id );
        // Default parameters
        $general_options = Options::getInstance( 'general', $blog_id );
        $general_options->saveOption( "post_type_post", "on" );
        $general_options->saveOption( "post_type_post_comments", "on" );
        $general_options->saveOption( "page_type_single", "on" );
        $general_options->saveOption( "id_method_cookie", "on" );
        $general_options->saveOption( "user_can_change_reaction", "on" );
        // Deafult images
        $source_path_with_end_slash = DA_REACTIONS_PATH . 'assets/icons/svg/_default/';
        $files                      = FileSystem::getFiles(
            $source_path_with_end_slash,
            array( 'svg', 'jpg', 'png', 'gif' )
        );
        switch_to_blog( $blog_id );
        FileSystem::copyImagesToUploadDir( $files, $source_path_with_end_slash );
        restore_current_blog();
    }
    /**
     * Deletes data when a blog is deleted
     * Attached to delete_blog hook
     *
     * @param integer $blog_id
     *
     * @since 1.0.0
     */
    public function removeDataForDeletedBlog__premium_only( $blog_id ) {
        // Database
        global $wpdb;
        $prefix = $wpdb->get_blog_prefix( $blog_id );
        Data::dropTables( $prefix );
    }
}
